/*
	Description : 	Preview Render, gives you global controls to lower render settings for preview rendering only
	Version     : 	1.2a
	Date			:	21.09.2007
	Author		:	Jerry Ylilammi

	Changelog	:
		1.2a
			- Fixed bug with resolution override
		1.2
			- Now works with Region Render
		1.1
			- Override controls for DMC
			- Resolution override works better now when render dialog is open
			- Support for VRay VFB resolution settings
*/

global previewRenderDialog
global temp = undefined
try(DestroyDialog previewRenderDialog)catch()

-- #############################################################################################
-- #################################/ Store previous settings  \################################
-- #############################################################################################

if previewRenderDialog != undefined then (
	temp = #(
		previewRenderDialog.res_o.checked,
		previewRenderDialog.res_w.value,
		previewRenderDialog.res_h.value,
		previewRenderDialog.lock.checked,
		previewRenderDialog.aa_o.checked,
		previewRenderDialog.aa_e.checked,
		previewRenderDialog.aa_q.value,
		previewRenderDialog.g_o.checked,
		previewRenderDialog.g_e.checked,
		previewRenderDialog.g_q.value,
		previewRenderDialog.rr_o.checked,
		previewRenderDialog.rr_e.checked,
		previewRenderDialog.rr_d.value,
      previewRenderDialog.s_o.checked,
      previewRenderDialog.s_e.checked,
      previewRenderDialog.s_q.value,
      previewRenderDialog.gi_o.checked,
      previewRenderDialog.gi_e.checked,
      previewRenderDialog.gi_i.selection,
      previewRenderDialog.gi_b.value,
      previewRenderDialog.gi_l.value,
		previewRenderDialog.disp_o.checked,
		previewRenderDialog.disp_e.checked,
		previewRenderDialog.disp_l.value,
		previewRenderDialog.dmc_o.checked,
		previewRenderDialog.dmc_a.value,
		previewRenderDialog.dmc_n.value
	)
)

-- #############################################################################################
-- #################################/ User Interface           \################################
-- #############################################################################################

rollout previewRenderDialog "VRay Preview Render" width:200 height:361 (

	-- #################### // Resolution \\ ####################

	checkbutton res_o "Resolution" pos:[0,0] width:200 height:16 checked:false
	button res_1 "1:1" pos:[2,18] width:20
	button res_2 "1:2" pos:[24,18] width:20
	button res_4 "1:4" pos:[46,18] width:20
	spinner res_w "" type:#Integer range:[1,10000,640] pos:[68,21] width:54
	spinner res_h "" type:#Integer range:[1,10000,480] pos:[124,21] width:54
	checkbutton lock "L" pos:[180,18] width:18 toolTip:"Lock Aspect Ration"

	-- #################### // Anti-aliasing \\ ####################

	checkbutton aa_o "Anti-aliasing" pos:[0,41] width:200 height:16 checked:true
	checkbox aa_e "Enabled" pos:[4,61]
	spinner aa_q "Quality" type:#Integer range:[-3,100,0] pos:[138,61] width:60

   -- #################### // Glossy \\ ####################

	checkbutton g_o "Glossy" pos:[0,82] width:200 height:16 checked:true
	checkbox g_e "Enabled" pos:[4,102]
	spinner g_q "Subdivs" type:#Integer range:[1,256,1] pos:[138,102] width:60

   -- #################### // Reflection / Refraction \\ ####################

	checkbutton rr_o "Reflection/Refraction" pos:[0,123] width:200 height:16 checked:true
	checkbox rr_e "Enabled" pos:[4,143]
	spinner rr_d "Depth" type:#Integer range:[1,256,1] pos:[138,143] width:60

   -- #################### // Shadows \\ ####################

	checkbutton s_o "Shadows" pos:[0,164] width:200 height:16 checked:true
	checkbox s_e "Enabled" pos:[4,184]
	spinner s_q "Subdivs" type:#Integer range:[1,256,1] pos:[138,184] width:60

   -- #################### // Global Illumination \\ ####################

	checkbutton gi_o "Global Illumination" pos:[0,205] width:200 height:16 checked:true
	checkbox gi_e "Enabled" pos:[4,225]
	label label1 "Irr. Map" pos:[80,229]
	dropdownlist gi_i "" pos:[119,225] width:80 items:#("Very Low", "Low", "Medium", "High", "Very High")
	spinner gi_b "Brute Force Subdivs" type:#Integer range:[1,256,0] pos:[98,249] width:100
	spinner gi_l "Light Cache Subdivs" type:#Integer range:[1,10000,1] pos:[98,269] width:100

   -- #################### // Displacement \\ ####################

	checkbutton disp_o "Displacement" pos:[0,290] width:200 height:16 checked:true
   checkbox disp_e "Enabled" pos:[4,310]
   spinner disp_l "Edge Length" type:#Float range:[0.1,100.0,0.1] pos:[98,310] width:100

   -- #################### // DMC \\ ####################

	checkbutton dmc_o "DMC" pos:[0,331] width:200 height:16 checked:true
	spinner dmc_a "Adaptive A." type:#Float range:[0.0,1.0,0.85] pos:[34,351] width:70
   spinner dmc_n "Noise T." type:#Float range:[0.0,1.0,0.007] pos:[128,351] width:70

   -- #################### // Render \\ ####################

	button preview "Render Preview" width:200 height:30 pos:[0,372]


-- #############################################################################################
-- #################################/ Variables                \################################
-- #############################################################################################

	local sectionPicked = 0 -- Right-clicked section
	local aspect = 1.333 -- Aspect ration
	local eMods = #() -- Array containing all edited modifiers (VRayDisplacementMod)
	local eMats = #() -- Array containing all edited materials (VRayMtl)
	local eLights = #() -- Array containing all edited lights (VRayShadow)
	local eModsEL = #() -- Old edge lengths for modifiers
	local eMatsSubdivs = #() -- Old subdivisions for materials
	local eLightsSubdivs = #() -- Old subdivisions for lights
	local rDialog = false -- State of render dialog
	-- Temporary storage for settings so they can be restored after render ends
	local temp_aa = undefined
	local temp_g = undefined
	local temp_rr = undefined
	local temp_s = undefined
	local temp_gi = undefined
	local temp_disp = undefined
	local temp_dmc = undefined
	local temp_res = undefined

-- #############################################################################################
-- #################################/ Auto resizze             \################################
-- #############################################################################################

	fn resize = (
		local offset = 0

		if res_o.checked then (
			res_1.visible = res_2.visible = res_4.visible = res_w.visible = res_h.visible = lock.visible = true
		) else (
			res_1.visible = res_2.visible = res_4.visible = res_w.visible = res_h.visible = lock.visible = false
			offset += 24
		)

      aa_o.pos = [aa_o.pos[1],41-offset]
		if aa_o.checked then (
			aa_e.visible = aa_q.visible = true
         aa_e.pos = [aa_e.pos[1],61-offset]
         aa_q.pos = [aa_q.pos[1],61-offset]
		) else (
         aa_e.visible = aa_q.visible = false
         offset += 24
		)
		
      g_o.pos = [g_o.pos[1],82-offset]
		if g_o.checked then (
			g_e.visible = g_q.visible = true
         g_e.pos = [g_e.pos[1],102-offset]
         g_q.pos = [g_q.pos[1],102-offset]
		) else (
         g_e.visible = g_q.visible = false
         offset += 24
		)

      rr_o.pos = [rr_o.pos[1],123-offset]
		if rr_o.checked then (
			rr_e.visible = rr_d.visible = true
         rr_e.pos = [rr_e.pos[1],143-offset]
         rr_d.pos = [rr_d.pos[1],143-offset]
		) else (
         rr_e.visible = rr_d.visible = false
         offset += 24
		)
		
      s_o.pos = [s_o.pos[1],164-offset]
		if s_o.checked then (
			s_e.visible = s_q.visible = true
         s_e.pos = [s_e.pos[1],184-offset]
         s_q.pos = [s_q.pos[1],184-offset]
		) else (
         s_e.visible = s_q.visible = false
         offset += 24
		)

      gi_o.pos = [gi_o.pos[1],205-offset]
		if gi_o.checked then (
			gi_e.visible = label1.visible = gi_i.visible = gi_b.visible = gi_l.visible = true
         gi_e.pos = [gi_e.pos[1],225-offset]
         label1.pos = [label1.pos[1],229-offset]
         gi_i.pos = [gi_i.pos[1],225-offset]
         gi_b.pos = [gi_b.pos[1],249-offset]
         gi_l.pos = [gi_l.pos[1],269-offset]
		) else (
         gi_e.visible = label1.visible = gi_i.visible = gi_b.visible = gi_l.visible = false
         offset += 68
		)
		
      disp_o.pos = [disp_o.pos[1],290-offset]
		if disp_o.checked then (
			disp_e.visible = disp_l.visible = true
         disp_e.pos = [disp_e.pos[1],310-offset]
         disp_l.pos = [disp_l.pos[1],310-offset]
		) else (
         disp_e.visible = disp_l.visible = false
         offset += 24
		)

      dmc_o.pos = [dmc_o.pos[1],331-offset]
		if dmc_o.checked then (
			dmc_a.visible = dmc_n.visible = true
         dmc_a.pos = [dmc_a.pos[1],351-offset]
         dmc_n.pos = [dmc_n.pos[1],351-offset]
		) else (
         dmc_a.visible = dmc_n.visible = false
         offset += 24
		)

		preview.pos = [preview.pos[1],372-offset]

		previewRenderDialog.height = 402-offset
	)

	on res_o changed val do resize()
	on aa_o changed val do resize()
	on g_o changed val do resize()
	on rr_o changed val do resize()
	on s_o changed val do resize()
	on gi_o changed val do resize()
	on disp_o changed val do resize()
	on dmc_o changed val do resize()

-- #############################################################################################
-- #################################/ Render Resolution         ################################
-- #############################################################################################

   fn setRes divisor = (
      if matchPattern ((classOf renderers.current) as String) pattern:"*V_Ray*" then (
			renderSceneDialog.commit()
			local vray = renderers.current

			if vray.output_getsetsfrommax then (
				res_h.value = renderHeight / divisor
				res_w.value = renderWidth / divisor
			) else (
				res_h.value = vray.output_height / divisor
				res_w.value = vray.output_width / divisor
			)
			aspect = res_w.value as Float / res_h.value as Float
		)
	)

	on res_1 pressed do setRes 1
	on res_2 pressed do setRes 2
	on res_4 pressed do setRes 4
	on res_h changed val do if lock.checked then res_w.value = val * aspect else aspect = res_w.value / val
	on res_w changed val do if lock.checked then res_h.value = val / aspect else aspect = val / res_h.value

-- #############################################################################################
-- #################################/ Presets                  \################################
-- #############################################################################################

	fn setPreset quality section:0 = (
		-- AA
		if section == 0 or section == 1 then (
			aa_o.checked = true
			aa_e.checked = true
			case quality of (
				1: aa_q.value = -2
				2: aa_q.value = 1
				3: aa_q.value = 2
				4: aa_q.value = 4
			)
		)

		-- Glossy
		if section == 0 or section == 2 then (
			g_o.checked = true
			g_e.checked = true
			case quality of (
				1: g_e.checked = false
				2: g_q.value = 4
				3: g_q.value = 8
				4: g_q.value = 16
			)
		)

		-- Reflection/Refraction
		if section == 0 or section == 3 then (
			rr_o.checked = true
			rr_e.checked = true
			case quality of (
				1: rr_d.value = 2
				2: rr_d.value = 5
				3: rr_d.value = 5
				4: rr_o.checked = false
			)
		)
		
		-- Shadows
		if section == 0 or section == 4 then (
			s_o.checked = true
			s_e.checked = true
			case quality of (
				1: s_q.value = 1
				2: s_q.value = 4
				3: s_q.value = 8
				4: s_q.value = 16
			)
		)

		-- Global Illumination
		if section == 0 or section == 5 then (
			gi_o.checked = true
			gi_e.checked = true
			case quality of (
				1: (	gi_i.selection = 1
				  	   gi_b.value = 4
				   	gi_l.value = 200 )
				2: (	gi_i.selection = 2
				  	   gi_b.value = 8
				   	gi_l.value = 400 )
				3: (	gi_i.selection = 3
				  	   gi_b.value = 12
				   	gi_l.value = 600 )
				4: (	gi_i.selection = 4
				  	   gi_b.value = 16
				   	gi_l.value = 800 )
			)
		)

		-- Displacement
		if section == 0 or section == 6 then (
			disp_o.checked = true
			disp_e.checked = true
			case quality of (
				1: disp_e.checked = false
				2: disp_l.value = 14.0
				3: disp_l.value = 8.0
				4: disp_o.checked = false
			)
		)

		-- DMC
		if section == 0 or section == 7 then (
			case quality of (
				1: ( dmc_a.value =  0.85
					dmc_n.value = 0.015 )
				2: ( dmc_a.value =  0.85
					dmc_n.value = 0.01 )
				3: ( dmc_a.value =  0.85
					dmc_n.value = 0.01 )
				4: ( dmc_a.value =  0.95
					dmc_n.value = 0.007 )
			)
		)
		resize()
	)

	fn closeAll = ( res_o.checked = aa_o.checked = g_o.checked = rr_o.checked = s_o.checked = gi_o.checked = disp_o.checked = dmc_o.checked = false
						 resize() )

	fn openAll = ( res_o.checked = aa_o.checked = g_o.checked = rr_o.checked = s_o.checked = gi_o.checked = disp_o.checked = dmc_o.checked = true
						 resize() )

	fn localPreview section = (
		sectionPicked = section
		RCMenu presetsMenu (
			menuItem low "Low"
			menuItem medium "Medium"
			menuItem high "High"
			menuItem veryHigh "Very High"
			separator sep1
			menuItem open "Open All"
			menuItem close "Close All"
			separator sep2
			subMenu "About" (
         	menuItem about1 "VRay Preview Render"
         	menuItem about2 "Version 1.2a"
         	menuItem about3 "Copyright  Jerry Ylilammi"
			)			

			on low picked do setPreset 1 section:previewRenderDialog.sectionPicked
			on medium picked do setPreset 2 section:previewRenderDialog.sectionPicked
			on high picked do setPreset 3 section:previewRenderDialog.sectionPicked
			on veryHigh picked do setPreset 4 section:previewRenderDialog.sectionPicked
			on open picked do previewRenderDialog.openAll()
			on close picked do previewRenderDialog.closeAll()
		)

		popUpMenu presetsMenu pos:mouse.screenpos
	)
	
	on aa_o rightclick do localPreview 1
	on g_o rightclick do localPreview 2
	on rr_o rightclick do localPreview 3
	on s_o rightclick do localPreview 4
	on gi_o rightclick do localPreview 5
	on disp_o rightclick do localPreview 6
	on dmc_o rightclick do localPreview 7

	on preview rightclick do (
		RCMenu presetsMenu (
			menuItem low "All: Low"
			menuItem medium "All: Medium"
			menuItem high "All: High"
			menuItem veryHigh "All: Very High"
			separator sep1
			menuItem open "Open All"
			menuItem close "Close All"
			separator sep2
			subMenu "About" (
         	menuItem about1 "VRay Preview Render"
         	menuItem about2 "Version 1.2a"
         	menuItem about3 "Copyright  Jerry Ylilammi"
			)

			on low picked do previewRenderDialog.setPreset 1
			on medium picked do previewRenderDialog.setPreset 2
			on high picked do previewRenderDialog.setPreset 3
			on veryHigh picked do previewRenderDialog.setPreset 4
			on open picked do previewRenderDialog.openAll()
			on close picked do previewRenderDialog.closeAll()
		)

		popUpMenu presetsMenu pos:mouse.screenpos
	)

-- #############################################################################################
-- #################################/ Node handling            \################################
-- #############################################################################################

	-- #################### // Displacement Modifiers \\ ####################
	fn editDispMods = (
		for i in objects do for j in i.modifiers where classOf j == VRayDisplacementMod and findItem eMods j == 0 do (
			append eMods j
			append eModsEL j.edgeLength
			if j.edgeLength < disp_l.value then j.edgeLength = disp_l.value
		)
	)

	fn restoreDispMods = (
		for i in 1 to eMods.count do eMods[i].edgeLength = eModsEL[i]
		eMods = #()
		eModsEL = #()
	)

	-- #################### // Materials \\ ####################
	fn getSubMaterials mat = (
		local mats = #(mat)
		for i in 1 to getNumSubMtls mat do
			join mats (getSubMaterials (getSubMtl mat i))
		return mats
	)

	fn editMaterials = (
		local allMats = #()
		for i in sceneMaterials where superClassOf i == material do
			join allMats (getSubMaterials i)
		for i in allMats where classOf i == VRayMtl and findItem eMats i == 0 do (
			append eMats i
			append eMatsSubdivs #(i.reflection_subdivs, i.refraction_subdivs)
			if i.reflection_subdivs > g_q.value then i.reflection_subdivs = g_q.value
			if i.refraction_subdivs > g_q.value then i.refraction_subdivs = g_q.value
		)
	)
	
	fn restoreMaterials = (
		for i in 1 to eMats.count do (
			eMats[i].reflection_subdivs = eMatsSubdivs[i][1]
			eMats[i].refraction_subdivs = eMatsSubdivs[i][2]
		)
		eMats = #()
		eMatsSubdivs = #()
	)

	-- #################### // Lights \\ ####################
	fn editLights = (
		for i in lights do
			if classOf i == VRayLight then (
				append eLights i
				append eLightsSubdivs i.subdivs
				if i.subdivs > s_q.value then i.subdivs = s_q.value
			) else if classOf i == VraySun then (
				append eLights i
				append eLightsSubdivs i.shadow_subdivs
				if i.shadow_subdivs > s_q.value then i.shadow_subdivs = s_q.value
			) else if isproperty i "shadowgenerator" then if classOf i.shadowgenerator == VrayShadow then (
				append eLights i.shadowgenerator
				append eLightsSubdivs i.shadowgenerator.subdivs
				if i.shadowgenerator.subdivs > s_q.value then i.shadowgenerator.subdivs = s_q.value
			)
	)

	fn restoreLights = (
		for i in 1 to eLights.count do
			if classOf eLights[i] == VraySun then
				eLights[i].shadow_subdivs = eLightsSubdivs[i]
			else
				eLights[i].subdivs = eLightsSubdivs[i]
		eLights = #()
		eLightsSubdivs = #()
	)

-- #############################################################################################
-- #################################/ Render Preview           \################################
-- #############################################################################################

	-- Alters settings and stores previous settings
	fn editSettings = (
		local vray = renderers.current

		-- AA
		if aa_o.checked then try(
			/*
				AA Quality
					-..0	Adaptive Subdivision, undersampling
					1		Fixed
					2..+	DMC
			*/
			temp_aa = #(
				vray.imageSampler_type,
				vray.fixedRate_subdivs,
				vray.twoLevel_baseSubdivs,
				vray.twoLevel_fineSubdivs,
				vray.adaptiveSubdivision_minRate,
				vray.adaptiveSubdivision_maxRate
			)

 			if aa_q.value == 1 or aa_e.checked == false then (
				vray.imageSampler_type = 0
				vray.fixedRate_subdivs = 1
			) else if aa_q.value < 1 then (
         	vray.imageSampler_type = 2
            vray.adaptiveSubdivision_minRate = aa_q.value
  				vray.adaptiveSubdivision_maxRate = 1
			) else (
				vray.imageSampler_type = 1
  				vray.adaptiveSubdivision_minRate = 1
  				vray.adaptiveSubdivision_maxRate = aa_q.value
			)
		)catch()
		
		-- Glossy
		if g_o.checked then try(
			temp_g = vray.options_glossyEffects
			if g_e.checked then (
				editMaterials()
			) else (
			  vray.options_glossyEffects = false
			)
		)catch()

		-- Reflection/Refraction
		if rr_o.checked then try(
			temp_rr = #(
				vray.options_reflectionRefraction,
				vray.options_limitDepth,
  				vray.options_maxDepth
  			)
  			if rr_e.checked then (
				vray.options_limitDepth = true
  				vray.options_maxDepth = rr_d.value
			) else (
         	vray.options_reflectionRefraction = false
			)
		)catch()

		-- Shadows
		if s_o.checked then try(
			temp_s = vray.options_shadows
			if s_e.checked then
				editLights()
			else
				vray.options_shadows = false
		)catch()

		-- Global Illumination
		if gi_o.checked then try(
			temp_gi = #(
				vray.gi_on,
				vray.gi_irradmap_minRate,
				vray.gi_irradmap_maxRate,
				vray.gi_irradmap_subdivs,
				vray.gi_irradmap_interpSamples,
				vray.gi_irradmap_colorThreshold,
				vray.gi_irradmap_normalThreshold,
				vray.gi_irradmap_distThreshold,
				vray.qmcgi_subdivs,
				vray.lightcache_subdivs
			)

			if gi_e.checked then (
				case gi_i.selection of (
	     			1: local s = #(-4, -3, 20, 20, 0.4, 0.3, 0.1)
					2: local s = #(-3, -3, 30, 20, 0.4, 0.3, 0.1)
					3: local s = #(-3, -1, 40, 30, 0.4, 0.3, 0.1)
					4: local s = #(-3, -1, 50, 40, 0.3, 0.1, 0.1)
					5: local s = #(-3,  0, 50, 40, 0.3, 0.1, 0.1)
				)
				vray.gi_irradmap_minRate = s[1]
				vray.gi_irradmap_maxRate = s[2]
				vray.gi_irradmap_subdivs = s[3]
				vray.gi_irradmap_interpSamples = s[4]
				vray.gi_irradmap_colorThreshold = s[5]
				vray.gi_irradmap_normalThreshold = s[6]
				vray.gi_irradmap_distThreshold = s[7]
				vray.qmcgi_subdivs = gi_b.value
				vray.lightcache_subdivs = gi_l.value
			) else (
				vray.gi_on = false
			)
		)catch()

		-- Displacement
		if disp_o.checked then try(
			temp_disp = #(
		    	vray.options_displacement,
		    	vray.displacement_edgeLength
		   )
		   if disp_e.checked then (
				if vray.displacement_edgeLength < disp_l.value then vray.displacement_edgeLength = disp_l.value
				editDispMods()
		   ) else (
				vray.options_displacement = false
			)
		)catch()

		-- DMC
		if dmc_o.checked then try(
			temp_dmc = #(
		    	vray.qmc_earlyTermination_amount,
		    	vray.qmc_earlyTermination_threshold
		   )
			vray.qmc_earlyTermination_amount = dmc_a.value
			vray.qmc_earlyTermination_threshold = dmc_n.value
		)catch()
		
		-- Resolution
		if res_o.checked then (
			rDialog = renderSceneDialog.isOpen()
			if rDialog then renderSceneDialog.close()

			temp_res = #(
				renderHeight,
				renderWidth,
				vray.output_height,
				vray.output_width
			)

			renderHeight = res_h.value
			renderWidth = res_w.value
			vray.output_height = res_h.value
         vray.output_width = res_w.value
		)
	)

	-- Restores all stored settings
	fn restoreSettings = (
		local vray = renderers.current
		-- AA
		if aa_o.checked then try(
			vray.imageSampler_type = temp_aa[1]
			vray.fixedRate_subdivs = temp_aa[2]
			vray.twoLevel_baseSubdivs = temp_aa[3]
			vray.twoLevel_fineSubdivs = temp_aa[4]
			vray.adaptiveSubdivision_minRate = temp_aa[5]
			vray.adaptiveSubdivision_maxRate = temp_aa[6]
		)catch()
		-- Glossy
		if g_o.checked then try(
			vray.options_glossyEffects = temp_g
			restoreMaterials()
		)catch()
		-- Reflection/Refraction
		if rr_o.checked then try(
			vray.options_reflectionRefraction = temp_rr[1]
			vray.options_limitDepth = temp_rr[2]
			vray.options_maxDepth = temp_rr[3]
		)catch()
		-- Shadows
		if s_o.checked then try(
			vray.options_shadows = temp_s
			restoreLights()
		)catch()
		-- Global Illumination
		if gi_o.checked then try(
			vray.gi_on = temp_gi[1]
			vray.gi_irradmap_minRate = temp_gi[2]
			vray.gi_irradmap_maxRate = temp_gi[3]
			vray.gi_irradmap_subdivs = temp_gi[4]
			vray.gi_irradmap_interpSamples = temp_gi[5]
			vray.gi_irradmap_colorThreshold = temp_gi[6]
			vray.gi_irradmap_normalThreshold = temp_gi[7]
			vray.gi_irradmap_distThreshold = temp_gi[8]
			vray.qmcgi_subdivs = temp_gi[9]
			vray.lightcache_subdivs = temp_gi[10]
		)catch()
		-- Displacement
		if disp_o.checked then try(
	    	vray.options_displacement = temp_disp[1]
	    	vray.displacement_edgeLength = temp_disp[2]
			restoreDispMods()
		)catch()
		-- DMC
		if dmc_o.checked then try(
	    	vray.qmc_earlyTermination_amount = temp_dmc[1]
	    	vray.qmc_earlyTermination_threshold = temp_dmc[2]
		)catch()
		-- Resolution
		if res_o.checked then (
			renderHeight = temp_res[1]
         renderWidth = temp_res[2]
			vray.output_height = temp_res[3]
         vray.output_width = temp_res[4]
			if rDialog then renderSceneDialog.open()
		)
	)

	fn renderPreview = if matchPattern ((classOf renderers.current) as String) pattern:"*V_Ray*" then (
		-- #################### // Alter settings \\ ####################
		temp_aa = temp_g = temp_rr = temp_s = temp_gi = temp_disp = temp_dmc = temp_res = undefined -- Reset stored settings if any
		editSettings()

		-- #################### // Render \\ ####################
		if getrendertype() == #region then try(max render last)catch()
		else try(max quick render)catch()

		-- #################### // Restore settings \\ ####################
		restoreSettings()
	)

	on preview pressed do renderPreview()
)

CreateDialog previewRenderDialog style:#(#style_toolwindow, #style_sysmenu)

-- #############################################################################################
-- #################################/ Restore previous settings\################################
-- #############################################################################################

if temp != undefined then (
	previewRenderDialog.res_o.checked = temp[1]
	previewRenderDialog.res_w.value = temp[2]
	previewRenderDialog.res_h.value = temp[3]
	previewRenderDialog.lock.checked = temp[4]
	previewRenderDialog.aa_o.checked = temp[5]
	previewRenderDialog.aa_e.checked = temp[6]
	previewRenderDialog.aa_q.value = temp[7]
	previewRenderDialog.g_o.checked = temp[8]
	previewRenderDialog.g_e.checked = temp[9]
	previewRenderDialog.g_q.value = temp[10]
	previewRenderDialog.rr_o.checked = temp[11]
	previewRenderDialog.rr_e.checked = temp[12]
	previewRenderDialog.rr_d.value = temp[13]
   previewRenderDialog.s_o.checked = temp[14]
   previewRenderDialog.s_e.checked = temp[15]
	previewRenderDialog.s_q.value = temp[16]
   previewRenderDialog.gi_o.checked = temp[17]
   previewRenderDialog.gi_e.checked = temp[18]
   previewRenderDialog.gi_i.selection = temp[19]
   previewRenderDialog.gi_b.value = temp[20]
   previewRenderDialog.gi_l.value = temp[21]
	previewRenderDialog.disp_o.checked = temp[22]
	previewRenderDialog.disp_e.checked = temp[23]
	previewRenderDialog.disp_l.value = temp[24]
	previewRenderDialog.dmc_o.checked = temp[25]
	previewRenderDialog.dmc_a.value = temp[26]
	previewRenderDialog.dmc_n.value = temp[27]
	previewRenderDialog.resize()
	temp = undefined
) else (
	previewRenderDialog.setPreset 2
)

/*
	Copyright  Jerry Ylilammi
*/